package teethferries.ui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import teethferries.model.*;

public class MainFrame extends JFrame implements ActionListener
{
	private static final long serialVersionUID = 1L;

	private Controller controller;

	private JLabel departureLabel, arrivalLabel, tratteLabel, outputLabel;
	private JTextArea output;
	private JComboBox<Porto> comboPortoPartenza;
	private JComboBox<Tratta> comboTratte;
	private JComboBox<Porto> comboPortoArrivo;
	@SuppressWarnings("unused")
	private Porto lastSelectedPartenza, lastSelectedArrivo;

	private Tratta lastSelectedTratta;  

	public MainFrame(MainController controller)
	{
		this.controller = controller;
		initGUI();
		pack();
		lastSelectedPartenza = null;
		lastSelectedArrivo = null;
	}
	
	private void initGUI()
	{
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setPreferredSize(new Dimension(340, 260));
		JPanel panel = new JPanel();
		
		departureLabel = new JLabel("Da");
		departureLabel.setFont(new Font("Times", Font.BOLD, 16));
		departureLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		panel.add(departureLabel);
		
		Porto[] porti = controller.elencoPorti().toArray(new Porto[0]);
		
		comboPortoPartenza = new JComboBox<>(porti);
		panel.add(comboPortoPartenza);
		comboPortoPartenza.addActionListener(this);

		arrivalLabel = new JLabel("A:");
		arrivalLabel.setFont(new Font("Times", Font.BOLD, 16));
		arrivalLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		panel.add(arrivalLabel);
		comboPortoArrivo = new JComboBox<>(porti);
		panel.add(comboPortoArrivo);
		comboPortoArrivo .addActionListener(this);
		
		tratteLabel = new JLabel("Tratte servite:");
		tratteLabel.setFont(new Font("Times", Font.BOLD, 16));
		tratteLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		panel.add(tratteLabel);
		comboTratte = new JComboBox<Tratta>(); // popolata dinamicamente
		comboTratte.setEditable(false);
		comboTratte.addActionListener(this);
		panel.add(comboTratte);

		outputLabel = new JLabel("Servizi disponibili:");
		outputLabel.setFont(new Font("Times", Font.BOLD, 16));
		outputLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		panel.add(outputLabel);
		output = new JTextArea(5,10);
		output.setEditable(false);
		panel.add(output);
		this.getContentPane().add(panel);

	}

	public void refreshComboTratte() {
		comboTratte.removeAllItems();
		for (Tratta t: controller.listaTratte()) {
			Porto da = t.getPortoPartenza();
			Porto a = t.getPortoArrivo();
			if (a==null || da==null) return; // salvaguardia
			// ora sono sicuramente non-nulli entrambi
			if (a.equalsOrANY((Porto)comboPortoArrivo.getSelectedItem()) && da.equalsOrANY((Porto)comboPortoPartenza.getSelectedItem())) {
				comboTratte.addItem(t);
			}
		}
	}

	public void mostraPercorsi(Tratta t) {
		output.setText("");
		for (Servizio s : controller.listaServizi()){
			if (s.getTratta().getId().equals(t.getId()))
				output.append(s.toString()+"\n");
		}
	}

	@Override
	public void actionPerformed(ActionEvent arg0)
	{
		Object source = arg0.getSource();
		
		if (source == comboPortoPartenza)	{
			lastSelectedPartenza = comboPortoPartenza.getItemAt(comboPortoPartenza.getSelectedIndex());
			refreshComboTratte();
		} else if (source == comboPortoArrivo)	{
			lastSelectedArrivo = comboPortoArrivo.getItemAt(comboPortoArrivo.getSelectedIndex());
			refreshComboTratte();
		}
		else if (source == comboTratte){
			lastSelectedTratta = comboTratte.getItemAt(comboTratte.getSelectedIndex());
			if (lastSelectedTratta == null) return;
			mostraPercorsi(lastSelectedTratta);
		}
		
	}

}
